﻿Option Explicit On
Option Strict On

Imports System.Text



Public Class frmFF3x

    Private clsFF3x_API As New FF3x_API.FF3x_API
    Public bytResponse() As Byte = Nothing

    Private Sub frmFF3x_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Dim intNumOfDevices As Integer

        intNumOfDevices = clsFF3x_API.GetNumOfDevices
        ' check if any FF3x device is present
        If intNumOfDevices = 0 Then
            MsgBox("No FF3x device found... exiting")
            Me.Close()
            Return
        End If
        Me.txtAboutNumOfDevices.Text = intNumOfDevices.ToString

        ' get 1st device path
        If clsFF3x_API.GetPath(1) = False Then
            MsgBox("Unable to locate FF3x device... exiting")
            Me.Close()
            Return
        End If
        Me.txtAbout1stDevicePath.Text = clsFF3x_API.Path

        ' connect to the device
        If clsFF3x_API.OpenComm = False Then
            MsgBox("Unable to open FF3x device... exiting")
            Me.Close()
            Return
        End If

        ' get chip type and silicon version
        If clsFF3x_API.GetChipInfo(bytResponse) = False Then
            PrintMessage("Error", "GetChipInfo - " & clsFF3x_API.LastError)
        Else
            If (bytResponse(2) = &HB) Then
                Select Case bytResponse(1)
                    Case &HF
                        Me.txtAbout1stDevice.Text = "FF32"
                        Me.txtAboutVersion.Text = CInt(bytResponse(3)) & "." & CInt(bytResponse(4))
                    Case &H10
                        Me.txtAbout1stDevice.Text = "FF34"
                        Me.txtAboutVersion.Text = CInt(bytResponse(3)) & "." & CInt(bytResponse(4))
                    Case Else
                        Me.txtAbout1stDevice.Text = "???"
                        Me.txtAboutVersion.Text = "???"
                End Select
            Else
                Me.txtAbout1stDevice.Text = "???"
                Me.txtAboutVersion.Text = "???"
            End If
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)

        cmbDigiOutPin.Text = "A1"
        cmbDigiOutPinValue.Text = "0"
        cmbDigiOutPinsBlock.Text = "A"
        cmbDigiInPin.Text = "B1"
        cmbDigiInPinsBlock.Text = "B"
        cmbPWMPin.Text = "A1"
        cmbAnalogInPin.Text = "B1"
        cmbSPICSPin.Text = "A1"
        cmbSPISCKPin.Text = "A2"
        cmbSPIMOSIPin.Text = "A3"
        cmbSPIMISOPin.Text = "A4"
        cmbSPIDevice.Text = "RAM 23K256"
        cmbSPIAction.Text = "Read"
        cmbI2CSCLPin.Text = "A5"
        cmbI2CSDAPin.Text = "A6"
        cmbI2CDevice.Text = "EEPROM 24LC512"
        cmbI2CAddress.Text = "0x50"
        cmbI2CAction.Text = "Read"
        cmb1WireDQPin.Text = "B1"
        cmb1WireDevice.Text = "DS18B20"
        cmb1WireAction.Text = "Get Temperature"
    End Sub

    Private Sub frmFF3x_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        Try
            clsFF3x_API.CloseComm()
        Catch ex As Exception
        End Try
    End Sub

    Private Sub cmdExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdExit.Click
        Me.Close()
    End Sub

    Private Sub PrintMessage(ByVal strCategory As String, ByVal strContent As String)
        If rtbStatus.TextLength > 0 Then
            rtbStatus.AppendText(vbCrLf)
        End If
        rtbStatus.SelectionStart = rtbStatus.Text.Length
        Select Case strCategory
            Case "Error"
                rtbStatus.SelectionColor = Color.Red
            Case "Sent"
                rtbStatus.SelectionColor = Color.Blue
            Case "Received"
                rtbStatus.SelectionColor = Color.DarkGreen
            Case Else
                rtbStatus.SelectionColor = Color.Orange
        End Select
        rtbStatus.AppendText(strCategory & ": ")
        rtbStatus.SelectionColor = Color.Black
        rtbStatus.AppendText(strContent)
    End Sub

    Private Sub cmdStatusClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdStatusClear.Click
        rtbStatus.Clear()
    End Sub

    Private Function ConvertPinName2PinBlock(ByVal strPin As String) As Byte
        Dim bytPin() As Byte = System.Text.Encoding.ASCII.GetBytes(strPin)
        Return CByte(bytPin(0) - &H37)
    End Function
    Private Function ConvertPinName2PinNumber(ByVal strPin As String) As Byte
        Dim bytPin() As Byte = System.Text.Encoding.ASCII.GetBytes(strPin)
        If bytPin.Length = 3 Then   ' two-digits pin number
            Return CByte((bytPin(1) - 39) + (bytPin(2) - 48))
        End If
        Return CByte(bytPin(1) - 48)
    End Function

    Private Function HexToByte(ByVal strIn As String) As Byte
        Return CByte(CInt("&H" & strIn.Substring(2, 2)))
    End Function


#Region " About "

    Private Sub cmdAboutAddressGet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAboutAddressGet.Click
        If clsFF3x_API.GetAddress(bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        Else
            Me.txtAboutAddress.Text = CInt(bytResponse(1)).ToString
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmdAboutAddressSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAboutAddressSet.Click
        If Not IsNumeric(txtAboutAddress.Text) Then
            PrintMessage("Error", "Numeric value is required")
            Return
        End If
        If (CInt(txtAboutAddress.Text) < 0) Or (CInt(txtAboutAddress.Text) > 127) Then
            PrintMessage("Error", "Numeric value must be between 0 and 127")
            Return
        End If
        If clsFF3x_API.SetAddress(CByte(txtAboutAddress.Text), bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmdAboutVendorGet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAboutVendorGet.Click
        If clsFF3x_API.GetVendor(bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        Else
            Me.txtAboutVendor.Text = System.Text.Encoding.ASCII.GetString(bytResponse, 1, bytResponse.Length - 2)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmdAboutVendorSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAboutVendorSet.Click
        Dim bytVendor() As Byte = System.Text.Encoding.ASCII.GetBytes(txtAboutVendor.Text)
        If clsFF3x_API.SetVendor(bytVendor, bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmdAboutProductGet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAboutProductGet.Click
        If clsFF3x_API.GetProduct(bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        Else
            Me.txtAboutProduct.Text = System.Text.Encoding.ASCII.GetString(bytResponse, 1, bytResponse.Length - 2)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmdAboutProductSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAboutProductSet.Click
        Dim bytProduct() As Byte = System.Text.Encoding.ASCII.GetBytes(txtAboutProduct.Text)
        If clsFF3x_API.SetProduct(bytProduct, bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmdAboutSerialGet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAboutSerialGet.Click
        If clsFF3x_API.GetSerialNumber(bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        Else
            Me.txtAboutSerial.Text = System.Text.Encoding.ASCII.GetString(bytResponse, 1, bytResponse.Length - 2)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmdAboutSerialSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAboutSerialSet.Click
        Dim bytSerial() As Byte = System.Text.Encoding.ASCII.GetBytes(txtAboutSerial.Text)
        If clsFF3x_API.SetSerialNumber(bytSerial, bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmdAboutReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAboutReset.Click
        clsFF3x_API.SetAddress(0, bytResponse)
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
        clsFF3x_API.SetVendor(Nothing, bytResponse)
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
        clsFF3x_API.SetProduct(Nothing, bytResponse)
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
        clsFF3x_API.SetSerialNumber(Nothing, bytResponse)
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
        MsgBox("Ok")
        txtAboutAddress.Text = ""
        txtAboutVendor.Text = ""
        txtAboutProduct.Text = ""
        txtAboutSerial.Text = ""
    End Sub

#End Region


#Region " Digital out "

    Private Sub cmdDigiOutPinSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDigiOutPinSet.Click
        If clsFF3x_API.SetDigitalOutput(ConvertPinName2PinBlock(cmbDigiOutPin.Text), ConvertPinName2PinNumber(cmbDigiOutPin.Text), CByte(cmbDigiOutPinValue.Text), bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmbDigiOutPinsBlock_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbDigiOutPinsBlock.SelectedIndexChanged
        Dim i As Integer
        dgwDigiOutPinsBlockSettings.Rows.Clear()
        If cmbDigiOutPinsBlock.SelectedIndex = 0 Then
            For i = 1 To 6
                dgwDigiOutPinsBlockSettings.Rows.Add()
                Dim cbcCell As DataGridViewCell = dgwDigiOutPinsBlockSettings.Rows(i - 1).Cells(0)
                cbcCell.Value = "A" & i
            Next
        Else
            For i = 1 To 12
                dgwDigiOutPinsBlockSettings.Rows.Add()
                Dim cbcCell As DataGridViewCell = dgwDigiOutPinsBlockSettings.Rows(i - 1).Cells(0)
                cbcCell.Value = "B" & i
            Next
        End If
    End Sub

    Private Sub dgwDigiOutPinsBlockSettings_CellClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles dgwDigiOutPinsBlockSettings.CellClick
        Select Case dgwDigiOutPinsBlockSettings.Columns(e.ColumnIndex).Index
            Case 1
                Dim cbcCell As DataGridViewCell = dgwDigiOutPinsBlockSettings.Rows(e.RowIndex).Cells(1)
                If CBool(cbcCell.Value) = True Then
                    cbcCell.Value = False
                    dgwDigiOutPinsBlockSettings.Rows(e.RowIndex).Cells(2) = New DataGridViewTextBoxCell
                    dgwDigiOutPinsBlockSettings.Rows(e.RowIndex).Cells(2).Style.BackColor = Color.FromKnownColor(KnownColor.Control)
                Else
                    cbcCell.Value = True
                    dgwDigiOutPinsBlockSettings.Rows(e.RowIndex).Cells(2) = New DataGridViewComboBoxCell
                    Dim cbcValue As DataGridViewComboBoxCell = CType(dgwDigiOutPinsBlockSettings.Rows(e.RowIndex).Cells(2), DataGridViewComboBoxCell)
                    cbcValue.Items.Add("0")
                    cbcValue.Items.Add("1")
                    cbcValue.Value = "0"
                    dgwDigiOutPinsBlockSettings.Rows(e.RowIndex).Cells(2).Style.BackColor = Color.FromKnownColor(KnownColor.Window)
                End If
            Case 2
                If CBool(dgwDigiOutPinsBlockSettings.Rows(e.RowIndex).Cells(1).Value) = True Then
                    Dim cbcValue As DataGridViewComboBoxCell = CType(dgwDigiOutPinsBlockSettings.Rows(e.RowIndex).Cells(2), DataGridViewComboBoxCell)
                    If cbcValue.Value.ToString = "0" Then
                        cbcValue.Value = "1"
                    Else
                        cbcValue.Value = "0"
                    End If
                End If
        End Select
    End Sub

    Private Sub cmdDigiOutPinsBlockSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDigiOutPinsBlockSet.Click
        Dim bytBlock As Byte
        Dim bytMask(1) As Byte
        Dim bytState(1) As Byte
        Dim intMask As Integer = 0
        Dim intState As Integer = 0

        If cmbDigiOutPinsBlock.SelectedIndex = 0 Then    ' block A
            bytBlock = CByte(&HA)
            bytMask(0) = 0
            bytState(0) = 0
            For i = 5 To 0 Step -1
                intMask *= 2
                intState *= 2
                Dim cbcCell As DataGridViewCell = dgwDigiOutPinsBlockSettings.Rows(i).Cells(1)
                Dim cbcValue As DataGridViewCell = dgwDigiOutPinsBlockSettings.Rows(i).Cells(2)
                If CBool(cbcCell.Value) = True Then
                    intMask += 1
                    intState+= CInt(cbcValue.Value) 
                End If
            Next
            bytMask(1) = CByte(intMask)
            bytState(1) = CByte(intState)
        Else    ' block B
            bytBlock = CByte(&HB)
            For i = 11 To 0 Step -1
                intMask *= 2
                intState *= 2
                Dim cbcCell As DataGridViewCell = dgwDigiOutPinsBlockSettings.Rows(i).Cells(1)
                Dim cbcValue As DataGridViewCell = dgwDigiOutPinsBlockSettings.Rows(i).Cells(2)
                If CBool(cbcCell.Value) = True Then
                    intMask += 1
                    intState += CInt(cbcValue.Value)
                End If
                If i = 8 Then
                    bytMask(0) = CByte(intMask)
                    bytState(0) = CByte(intState)
                    intMask = 0
                    intState = 0
                End If
                bytMask(1) = CByte(intMask)
                bytState(1) = CByte(intState)
            Next
        End If

        If clsFF3x_API.SetBlockDigitalOutputs(bytBlock, bytMask, bytState, bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

#End Region


#Region " Digital in "

    Private Sub cmdDigiInPinSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDigiInPinSet.Click
        If clsFF3x_API.GetDigitalInput(ConvertPinName2PinBlock(cmbDigiInPin.Text), ConvertPinName2PinNumber(cmbDigiInPin.Text), bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        Else
            txtDigiInPinValue.Text = (bytResponse(1)).ToString
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmbDigiInPinsBlock_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbDigiInPinsBlock.SelectedIndexChanged
        Dim i As Integer
        dgwDigiInPinsBlockSettings.Rows.Clear()
        If cmbDigiInPinsBlock.SelectedIndex = 0 Then
            For i = 1 To 6
                dgwDigiInPinsBlockSettings.Rows.Add()
                Dim cbcCell As DataGridViewCell = dgwDigiInPinsBlockSettings.Rows(i - 1).Cells(0)
                cbcCell.Value = "A" & i
            Next
        Else
            For i = 1 To 12
                dgwDigiInPinsBlockSettings.Rows.Add()
                Dim cbcCell As DataGridViewCell = dgwDigiInPinsBlockSettings.Rows(i - 1).Cells(0)
                cbcCell.Value = "B" & i
            Next
        End If
    End Sub

    Private Sub dgwDigiInPinsBlockSettings_CellClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles dgwDigiInPinsBlockSettings.CellClick
        If dgwDigiInPinsBlockSettings.Columns(e.ColumnIndex).Index = 1 Then
            Dim cbcCell As DataGridViewCell = dgwDigiInPinsBlockSettings.Rows(e.RowIndex).Cells(1)
            If CBool(cbcCell.Value) = True Then
                cbcCell.Value = False
            Else
                cbcCell.Value = True
            End If
        End If
    End Sub

    Private Sub cmbDigiInPinsBlockGet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDigiInPinsBlockGet.Click
        Dim bytBlock As Byte
        Dim bytMask(1) As Byte
        Dim intMask As Integer = 0

        If cmbDigiInPinsBlock.SelectedIndex = 0 Then    ' block A
            bytBlock = CByte(&HA)
            bytMask(0) = 0
            For i = 5 To 0 Step -1
                intMask *= 2
                Dim cbcCell As DataGridViewCell = dgwDigiInPinsBlockSettings.Rows(i).Cells(1)
                If CBool(cbcCell.Value) = True Then
                    intMask += 1
                End If
            Next
            bytMask(1) = CByte(intMask)
        Else    ' block B
            bytBlock = CByte(&HB)
            For i = 11 To 0 Step -1
                intMask *= 2
                Dim cbcCell As DataGridViewCell = dgwDigiInPinsBlockSettings.Rows(i).Cells(1)
                If CBool(cbcCell.Value) = True Then
                    intMask += 1
                End If
                If i = 8 Then
                    bytMask(0) = CByte(intMask)
                    intMask = 0
                End If
            Next
            bytMask(1) = CByte(intMask)
        End If

        If clsFF3x_API.GetBlockDigitalInputs(bytBlock, bytMask, bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        Else
            intMask = 1
            If cmbDigiInPinsBlock.SelectedIndex = 0 Then    ' block A
                For i = 0 To 5
                    Dim cbcMask As DataGridViewCell = dgwDigiInPinsBlockSettings.Rows(i).Cells(1)
                    Dim cbcValue As DataGridViewCell = dgwDigiInPinsBlockSettings.Rows(i).Cells(2)
                    If CBool(cbcMask.Value) = True Then
                        If (bytResponse(2) And intMask) = 0 Then
                            cbcValue.Value = 0
                        Else
                            cbcValue.Value = 1
                        End If
                    Else
                        cbcValue.Value = ""
                    End If
                    intMask *= 2
                Next
            Else        ' block B
                For i = 0 To 11
                    Dim cbcMask As DataGridViewCell = dgwDigiInPinsBlockSettings.Rows(i).Cells(1)
                    Dim cbcValue As DataGridViewCell = dgwDigiInPinsBlockSettings.Rows(i).Cells(2)
                    If CBool(cbcMask.Value) = True Then
                        If (bytResponse(2 - (i \ 8)) And intMask) = 0 Then
                            cbcValue.Value = 0
                        Else
                            cbcValue.Value = 1
                        End If
                    Else
                        cbcValue.Value = ""
                    End If
                    intMask *= 2
                    If i = 7 Then
                        intMask = 1
                    End If
                Next
            End If
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

#End Region


#Region " PWM "

    Private Sub cmdPWMPinSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPWMPinSet.Click
        If clsFF3x_API.SetPWMOutput(ConvertPinName2PinBlock(cmbPWMPin.Text), ConvertPinName2PinNumber(cmbPWMPin.Text), _
                    CByte(trbPWMPinValue.Value), bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        Else
            txtDigiInPinValue.Text = (bytResponse(1)).ToString
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub trbPWMPinValue_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles trbPWMPinValue.ValueChanged
        txtPWMPinValue.Text = trbPWMPinValue.Value & "%"
    End Sub

#End Region


#Region " Analog in "

    Private Sub cmdAnalogInPinGet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAnalogInPinGet.Click
        If clsFF3x_API.GetAnalogInput(ConvertPinName2PinBlock(cmbAnalogInPin.Text), ConvertPinName2PinNumber(cmbAnalogInPin.Text), bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        Else
            Dim dblValue As Double = (((bytResponse(2) * 256) + bytResponse(3)) / 1023)
            If bytResponse(1) = CByte(&H33) Then
                dblValue *= 3.3
            Else
                dblValue *= 5
            End If
            txtAnalogInPinValue.Text = dblValue.ToString("F3") & "V"
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

#End Region


#Region " SPI "

    Private Sub cmbSPIPinsSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbSPIPinsSet.Click
        If clsFF3x_API.SetSPIPins(ConvertPinName2PinBlock(cmbSPICSPin.Text), ConvertPinName2PinNumber(cmbSPICSPin.Text), _
                ConvertPinName2PinBlock(cmbSPISCKPin.Text), ConvertPinName2PinNumber(cmbSPISCKPin.Text), _
                ConvertPinName2PinBlock(cmbSPIMOSIPin.Text), ConvertPinName2PinNumber(cmbSPIMOSIPin.Text), _
                ConvertPinName2PinBlock(cmbSPIMISOPin.Text), ConvertPinName2PinNumber(cmbSPIMISOPin.Text), _
                bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmbSPIAction_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbSPIAction.SelectedIndexChanged
        If cmbSPIAction.SelectedIndex = 0 Then
            txtSPIDataLen.Enabled = True
            txtSPIDataLen.Text = "1"
            txtSPIData.Enabled = False
        Else
            txtSPIDataLen.Enabled = False
            txtSPIDataLen.Text = ""
            txtSPIData.Enabled = True
        End If
        txtSPIData.Text = ""
    End Sub

    Private Sub cmdSPIExecute_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSPIExecute.Click
        If Not IsNumeric(txtSPILocation.Text) Then
            PrintMessage("Error", "Numeric value is required for Memory Location")
            Return
        End If
        If (CInt(txtSPILocation.Text) < 0) Or (CInt(txtSPILocation.Text) > 32767) Then  ' applicable to RAM chip 23K256
            PrintMessage("Error", "Memory Location value for this chip must be between 0 and 32767")
            Return
        End If

        If cmbSPIAction.SelectedIndex = 0 Then  ' read

            ' make sure the 23K256 is Sequential Operation
            Dim bytWriteData(1) As Byte
            bytWriteData(0) = &H1   ' write to Status register
            bytWriteData(1) = &H41  ' Sequential operation
            clsFF3x_API.WriteSPI(2, bytWriteData, bytResponse)

            Me.txtSPIData.Text = ""
            If Not IsNumeric(txtSPIDataLen.Text) Then
                PrintMessage("Error", "Numeric value is required for Data Length ")
                Return
            End If
            If (CInt(txtSPIDataLen.Text) < 0) Or (CInt(txtSPIDataLen.Text) > 60) Then
                PrintMessage("Error", "Data Length value must be between 0 and 60")
                Return
            End If

            ReDim bytWriteData(2)
            bytWriteData(0) = &H3    ' RAM read command
            bytWriteData(1) = CByte(CInt(txtSPILocation.Text) \ 256)     ' memory location, high byte
            bytWriteData(2) = CByte(CInt(txtSPILocation.Text) Mod 256)   ' memory location, low byte
            If clsFF3x_API.ReadSPI(3, CByte(txtSPIDataLen.Text), bytWriteData, bytResponse) = False Then
                PrintMessage("Error", clsFF3x_API.LastError)
            Else
                Me.txtSPIData.Text = System.Text.Encoding.ASCII.GetString(bytResponse, 2, bytResponse.Length - 2)
            End If
            PrintMessage("Sent", clsFF3x_API.LastDataSent)
            PrintMessage("Received", clsFF3x_API.LastDataReceived)
        Else        ' write

            ' make sure the 23K256 is in Sequential Operation
            Dim bytWriteData(1) As Byte
            bytWriteData(0) = &H1   ' write to Status register
            bytWriteData(1) = &H41  ' Sequential operation
            clsFF3x_API.WriteSPI(2, bytWriteData, bytResponse)

            Dim bytContent() As Byte = System.Text.Encoding.ASCII.GetBytes(txtSPIData.Text)
            Dim bytData(bytContent.Length + 2) As Byte
            Dim i As Integer
            bytData(0) = &H2    ' RAM write command
            bytData(1) = CByte(CInt(txtSPILocation.Text) \ 256)     ' memory location, high byte
            bytData(2) = CByte(CInt(txtSPILocation.Text) Mod 256)   ' memory location, low byte
            For i = 0 To bytContent.Length - 1     ' fill data payload
                bytData(i + 3) = bytContent(i)
            Next
            If clsFF3x_API.WriteSPI(CByte(bytData.Length), bytData, bytResponse) = False Then
                PrintMessage("Error", clsFF3x_API.LastError)
            End If
            PrintMessage("Sent", clsFF3x_API.LastDataSent)
            PrintMessage("Received", clsFF3x_API.LastDataReceived)
        End If
    End Sub

#End Region


#Region " I2C "

    Private Sub cmdI2CPinsSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdI2CPinsSet.Click
        If clsFF3x_API.SetI2CPins(ConvertPinName2PinBlock(cmbI2CSCLPin.Text), ConvertPinName2PinNumber(cmbI2CSCLPin.Text), _
                ConvertPinName2PinBlock(cmbI2CSDAPin.Text), ConvertPinName2PinNumber(cmbI2CSDAPin.Text), _
                bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmbI2CAction_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbI2CAction.SelectedIndexChanged
        If cmbI2CAction.SelectedIndex = 0 Then
            txtI2CDataLen.Enabled = True
            txtI2CDataLen.Text = "1"
            txtI2CData.Enabled = False
        Else
            txtI2CDataLen.Enabled = False
            txtI2CDataLen.Text = ""
            txtI2CData.Enabled = True
        End If
        txtI2CData.Text = ""
    End Sub

    Private Sub cmdI2CExecute_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdI2CExecute.Click
        If Not IsNumeric(txtI2CLocation.Text) Then
            PrintMessage("Error", "Numeric value is required for Memory Location")
            Return
        End If
        If (CInt(txtI2CLocation.Text) < 0) Or (CInt(txtI2CLocation.Text) > 65535) Then  ' applicable to EEPROM chip 24LC512
            PrintMessage("Error", "Memory Location value for this chip must be between 0 and 65535")
            Return
        End If

        If cmbI2CAction.SelectedIndex = 0 Then  ' read
            Me.txtI2CData.Text = ""
            If Not IsNumeric(txtI2CDataLen.Text) Then
                PrintMessage("Error", "Numeric value is required for Data Length ")
                Return
            End If
            If (CInt(txtI2CDataLen.Text) < 0) Or (CInt(txtI2CDataLen.Text) > 60) Then
                PrintMessage("Error", "Data Length value must be between 0 and 60")
                Return
            End If

            ' EEPROM 24LC512 reading is performed in two steps - we need to set memory location first (by performing write) and then we read data
            '
            ' set memory location
            Dim bytData(2) As Byte
            bytData(0) = CByte(HexToByte(cmbI2CAddress.Text) * 2)   ' merge address bits and write bit (0)
            bytData(1) = CByte(CInt(txtI2CLocation.Text) \ 256)     ' memory location, high byte
            bytData(2) = CByte(CInt(txtI2CLocation.Text) Mod 256)   ' memory location, low byte
            If clsFF3x_API.WriteI2C(CByte(bytData.Length), bytData, bytResponse) = False Then
                PrintMessage("Error", clsFF3x_API.LastError)
            End If
            PrintMessage("Sent", clsFF3x_API.LastDataSent)
            PrintMessage("Received", clsFF3x_API.LastDataReceived)

            ' read data
            Dim bytWriteData(0) As Byte
            bytWriteData(0) = CByte((HexToByte(cmbI2CAddress.Text) * 2) + 1)   ' merge address bits and read bit (1)
            If clsFF3x_API.ReadI2C(1, CByte(txtI2CDataLen.Text), bytWriteData, bytResponse) = False Then
                PrintMessage("Error", clsFF3x_API.LastError)
            Else
                Me.txtI2CData.Text = System.Text.Encoding.ASCII.GetString(bytResponse, 2, bytResponse.Length - 2)
            End If
            PrintMessage("Sent", clsFF3x_API.LastDataSent)
            PrintMessage("Received", clsFF3x_API.LastDataReceived)
        Else        ' write
            Dim bytContent() As Byte = System.Text.Encoding.ASCII.GetBytes(txtI2CData.Text)
            Dim bytData(bytContent.Length + 2) As Byte
            Dim i As Integer
            bytData(0) = CByte(HexToByte(cmbI2CAddress.Text) * 2)   ' merge address bits and write bit (0)
            bytData(1) = CByte(CInt(txtI2CLocation.Text) \ 256)     ' memory location, high byte
            bytData(2) = CByte(CInt(txtI2CLocation.Text) Mod 256)   ' memory location, low byte
            For i = 0 To bytContent.Length - 1     ' fill data payload
                bytData(i + 3) = bytContent(i)
            Next
            If clsFF3x_API.WriteI2C(CByte(bytData.Length), bytData, bytResponse) = False Then
                PrintMessage("Error", clsFF3x_API.LastError)
            End If
            PrintMessage("Sent", clsFF3x_API.LastDataSent)
            PrintMessage("Received", clsFF3x_API.LastDataReceived)
        End If
    End Sub

#End Region


#Region " 1-Wire "

    Private Sub cmd1WireDQPinSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmd1WireDQPinSet.Click
        If clsFF3x_API.Set1WirePin(ConvertPinName2PinBlock(cmb1WireDQPin.Text), ConvertPinName2PinNumber(cmb1WireDQPin.Text), _
                bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmd1WireExecute_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmd1WireExecute.Click
        txt1WireResponse.Text = ""
        If cmb1WireAction.SelectedIndex = 0 Then    ' read temperature
            Dim bytWriteData(1) As Byte
            bytWriteData(0) = CByte(&HCC)   ' DS18B20/DS18S20 command: SKIP ROM
            bytWriteData(1) = CByte(&H44)   ' DS18B20/DS18S20 command: CONVERT T
            If clsFF3x_API.Write1Wire(CByte(bytWriteData.Length), bytWriteData, bytResponse) = False Then
                PrintMessage("Error", clsFF3x_API.LastError)
            End If
            PrintMessage("Sent", clsFF3x_API.LastDataSent)
            PrintMessage("Received", clsFF3x_API.LastDataReceived)

            Threading.Thread.Sleep(1000) ' waiting 1s for the temperature conversion to complete

            bytWriteData(0) = CByte(&HCC)   ' DS18B20/DS18S20 command: SKIP ROM
            bytWriteData(1) = CByte(&HBE)   ' DS18B20/DS18S20 command: READ SCRATCHPAD
            If clsFF3x_API.Read1Wire(CByte(bytWriteData.Length), 9, bytWriteData, bytResponse) = False Then
                PrintMessage("Error", clsFF3x_API.LastError)
            Else
                Dim dblValue As Double
                If cmb1WireDevice.SelectedIndex = 0 Then    ' DS18B20 -> 12-bit resolution
                    If bytResponse(3) < 128 Then  ' positive temperature
                        dblValue = ((bytResponse(3) * 256) + bytResponse(2)) / 16
                    Else        ' negative temperature
                        dblValue = ((65536 - ((bytResponse(3) * 256) + bytResponse(2))) / 16) * -1
                    End If
                    txt1WireResponse.Text = dblValue.ToString("F2") & "°C"
                Else    ' DS18S20 -> 9-bit resolution
                    If bytResponse(3) = 0 Then  ' positive temperature
                        dblValue = bytResponse(2) / 2
                    Else        ' negative temperature
                        dblValue = 256 - (bytResponse(2) / 2)
                    End If
                    txt1WireResponse.Text = dblValue.ToString("F1") & "°C"
                End If
            End If
            PrintMessage("Sent", clsFF3x_API.LastDataSent)
            PrintMessage("Received", clsFF3x_API.LastDataReceived)
        Else        ' read serial number
            Dim bytWriteData(0) As Byte
            Dim i As Integer
            bytWriteData(0) = CByte(&H33)   ' DS18B20/DS18S20 command: READ ROM
            If clsFF3x_API.Read1Wire(CByte(bytWriteData.Length), 8, bytWriteData, bytResponse) = False Then
                PrintMessage("Error", clsFF3x_API.LastError)
            Else
                For i = 2 To 9
                    txt1WireResponse.Text &= (Convert.ToString(bytResponse(i), 16).PadLeft(2, "0"c)).ToUpper
                Next
            End If
            PrintMessage("Sent", clsFF3x_API.LastDataSent)
            PrintMessage("Received", clsFF3x_API.LastDataReceived)
        End If
    End Sub

    Private Sub cmd1WireReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmd1WireReset.Click
        If clsFF3x_API.Reset1Wire(bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmd1WireWriteBit1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmd1WireWriteBit1.Click
        If clsFF3x_API.WriteBit1Wire(1, bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmd1WireWriteBit0_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmd1WireWriteBit0.Click
        If clsFF3x_API.WriteBit1Wire(0, bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

    Private Sub cmd1WireReadBit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmd1WireReadBit.Click
        If clsFF3x_API.ReadBit1Wire(bytResponse) = False Then
            PrintMessage("Error", clsFF3x_API.LastError)
        End If
        PrintMessage("Sent", clsFF3x_API.LastDataSent)
        PrintMessage("Received", clsFF3x_API.LastDataReceived)
    End Sub

#End Region


End Class
